@@ -6,8 +6,14 @@ use strict;
 use vars qw( $VERSION @ISA );
 
 BEGIN {
-    $VERSION = '1.27_01';
+    $VERSION = '1.31_01';
     @ISA     = qw( Archive::Zip );
+
+    if ( $^O eq 'MSWin32' ) {
+        require Win32;
+        require Encode;
+        Encode->import( qw{ decode_utf8 } );
+    }
 }
 
 use Archive::Zip qw(
@@ -19,7 +25,7 @@ use Archive::Zip qw(
 );
 
 use Time::Local ();
-use Compress::Zlib qw( Z_OK Z_STREAM_END MAX_WBITS );
+use Compress::Raw::Zlib qw( Z_OK Z_STREAM_END MAX_WBITS );
 use File::Path;
 use File::Basename;
 
@@ -45,19 +51,52 @@ sub _newFromZipFile {
 
 sub newFromString {
     my $class = shift;
-    my $self  = $class->STRINGMEMBERCLASS->_newFromString(@_);
+
+    my ( $stringOrStringRef, $fileName );
+    if ( ref( $_[0] ) eq 'HASH' ) {
+        $stringOrStringRef = $_[0]->{string};
+        $fileName          = $_[0]->{zipName};
+    }
+    else {
+        ( $stringOrStringRef, $fileName ) = @_;
+    }
+
+    my $self  = $class->STRINGMEMBERCLASS->_newFromString( $stringOrStringRef,
+        $fileName );
     return $self;
 }
 
 sub newFromFile {
     my $class = shift;
-    my $self  = $class->NEWFILEMEMBERCLASS->_newFromFileNamed(@_);
+
+    my ( $fileName, $zipName );
+    if ( ref( $_[0] ) eq 'HASH' ) {
+        $fileName = $_[0]->{fileName};
+        $zipName  = $_[0]->{zipName};
+    }
+    else {
+        ( $fileName, $zipName ) = @_;
+    }
+
+    my $self = $class->NEWFILEMEMBERCLASS->_newFromFileNamed( $fileName,
+      $zipName );
     return $self;
 }
 
 sub newDirectoryNamed {
     my $class = shift;
-    my $self  = $class->DIRECTORYMEMBERCLASS->_newNamed(@_);
+
+    my ( $directoryName, $newName );
+    if ( ref( $_[0] ) eq 'HASH' ) {
+        $directoryName = $_[0]->{directoryName};
+        $newName       = $_[0]->{zipName};
+    }
+    else {
+        ( $directoryName, $newName ) = @_;
+    }
+
+    my $self  = $class->DIRECTORYMEMBERCLASS->_newNamed( $directoryName,
+        $newName );
     return $self;
 }
 
@@ -106,9 +145,15 @@ sub versionMadeBy {
 }
 
 sub fileAttributeFormat {
-    ( $#_ > 0 )
-      ? ( $_[0]->{'fileAttributeFormat'} = $_[1] )
-      : $_[0]->{'fileAttributeFormat'};
+    my $self = shift;
+
+    if (@_) {
+        $self->{fileAttributeFormat} = ( ref( $_[0] ) eq 'HASH' )
+        ? $_[0]->{format} : $_[0];
+    }
+    else {
+        return $self->{fileAttributeFormat};
+    }
 }
 
 sub versionNeededToExtract {
@@ -128,6 +173,10 @@ sub bitFlag {
     } elsif ( $self->desiredCompressionLevel == 8 || $self->desiredCompressionLevel == 9 ) {
         $self->{'bitFlag'} = DEFLATING_COMPRESSION_MAXIMUM;
     }
+
+    if ($Archive::Zip::UNICODE) {
+        $self->{'bitFlag'} = $self->{'bitFlag'} | 0x0800;
+    }
     $self->{'bitFlag'};
 }
 
@@ -136,8 +185,9 @@ sub compressionMethod {
 }
 
 sub desiredCompressionMethod {
-    my $self                        = shift;
-    my $newDesiredCompressionMethod = shift;
+    my $self = shift;
+    my $newDesiredCompressionMethod =
+      ( ref( $_[0] ) eq 'HASH' ) ? shift->{compressionMethod} : shift;
     my $oldDesiredCompressionMethod = $self->{'desiredCompressionMethod'};
     if ( defined($newDesiredCompressionMethod) ) {
         $self->{'desiredCompressionMethod'} = $newDesiredCompressionMethod;
@@ -153,8 +203,9 @@ sub desiredCompressionMethod {
 }
 
 sub desiredCompressionLevel {
-    my $self                       = shift;
-    my $newDesiredCompressionLevel = shift;
+    my $self = shift;
+    my $newDesiredCompressionLevel =
+      ( ref( $_[0] ) eq 'HASH' ) ? shift->{compressionLevel} : shift;
     my $oldDesiredCompressionLevel = $self->{'desiredCompressionLevel'};
     if ( defined($newDesiredCompressionLevel) ) {
         $self->{'desiredCompressionLevel'}  = $newDesiredCompressionLevel;
@@ -307,8 +358,11 @@ sub _mapPermissionsToUnix {
 sub unixFileAttributes {
     my $self     = shift;
     my $oldPerms = $self->_mapPermissionsToUnix;
+
+    my $perms;
     if ( @_ ) {
-        my $perms = shift;
+        $perms = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{attributes} : $_[0];
+
         if ( $self->isDirectory ) {
             $perms &= ~FILE_ATTRIB;
             $perms |= DIRECTORY_ATTRIB;
@@ -318,17 +372,32 @@ sub unixFileAttributes {
         }
         $self->{externalFileAttributes} = $self->_mapPermissionsFromUnix($perms);
     }
+
     return $oldPerms;
 }
 
 sub localExtraField {
-    ( $#_ > 0 )
-      ? ( $_[0]->{'localExtraField'} = $_[1] )
-      : $_[0]->{'localExtraField'};
+    my $self = shift;
+
+    if (@_) {
+        $self->{localExtraField} = ( ref( $_[0] ) eq 'HASH' )
+          ? $_[0]->{field} : $_[0];
+    }
+    else {
+        return $self->{localExtraField};
+    }
 }
 
 sub cdExtraField {
-    ( $#_ > 0 ) ? ( $_[0]->{'cdExtraField'} = $_[1] ) : $_[0]->{'cdExtraField'};
+    my $self = shift;
+
+    if (@_) {
+        $self->{cdExtraField} = ( ref( $_[0] ) eq 'HASH' )
+          ? $_[0]->{field} : $_[0];
+    }
+    else {
+        return $self->{cdExtraField};
+    }
 }
 
 sub extraFields {
@@ -337,9 +406,15 @@ sub extraFields {
 }
 
 sub fileComment {
-    ( $#_ > 0 )
-      ? ( $_[0]->{'fileComment'} = pack( 'C0a*', $_[1] ) )
-      : $_[0]->{'fileComment'};
+    my $self = shift;
+
+    if (@_) {
+        $self->{fileComment} = ( ref( $_[0] ) eq 'HASH' )
+          ? pack( 'C0a*', $_[0]->{comment} ) : pack( 'C0a*', $_[0] );
+    }
+    else {
+        return $self->{fileComment};
+    }
 }
 
 sub hasDataDescriptor {
@@ -380,7 +455,7 @@ sub isTextFile {
     my $self = shift;
     my $bit  = $self->internalFileAttributes() & IFA_TEXT_FILE_MASK;
     if (@_) {
-        my $flag = shift;
+        my $flag = ( ref( $_[0] ) eq 'HASH' ) ? shift->{flag} : shift;
         $self->{'internalFileAttributes'} &= ~IFA_TEXT_FILE_MASK;
         $self->{'internalFileAttributes'} |=
           ( $flag ? IFA_TEXT_FILE: IFA_BINARY_FILE );
@@ -402,11 +477,13 @@ sub isBinaryFile {
 
 sub extractToFileNamed {
     my $self = shift;
-    my $name = shift;    # local FS name
+
+    # local FS name
+    my $name = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{name} : $_[0];
     $self->{'isSymbolicLink'} = 0;
 
     # Check if the file / directory is a symbolic link or not
-    if ( $self->{'externalFileAttributes'} == 2717843456 ) {
+    if ( $self->{'externalFileAttributes'} == 0xA1FF0000 ) {
         $self->{'isSymbolicLink'} = 1;
         $self->{'newName'} = $name;
         my ( $status, $fh ) = _newFileHandle( $name, 'r' );
@@ -415,8 +492,23 @@ sub extractToFileNamed {
     } else {
         #return _writeSymbolicLink($self, $name) if $self->isSymbolicLink();
         return _error("encryption unsupported") if $self->isEncrypted();
-        mkpath( dirname($name) );    # croaks on error
-        my ( $status, $fh ) = _newFileHandle( $name, 'w' );
+        
+        my ( $status, $fh );
+        if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+            $name = decode_utf8( Win32::GetFullPathName($name) );
+            mkpath_win32($name);
+            if ( $self->{'compressedSize'} == 0 ) {
+                return;
+            }
+            else {
+                Win32::CreateFile($name);
+            }
+            ( $status, $fh ) = _newFileHandle( Win32::GetANSIPathName($name), 'w' );
+        }
+        else {
+            mkpath( dirname($name) );    # croaks on error
+            ( $status, $fh ) = _newFileHandle( $name, 'w' );
+        }
         return _ioError("Can't open file $name for write") unless $status;
         my $retval = $self->extractToFileHandle($fh);
         $fh->close();
@@ -427,6 +519,19 @@ sub extractToFileNamed {
     }
 }
 
+sub mkpath_win32 {
+    my $path = shift;
+    use File::Spec;
+
+    my ( $volume, @path ) = File::Spec->splitdir($path);
+    $path = File::Spec->catfile( $volume, shift @path );
+    pop @path;
+    while (@path) {
+        $path = File::Spec->catfile( $path, shift @path );
+        Win32::CreateDirectory($path);
+    }
+}
+
 sub _writeSymbolicLink {
     my $self = shift;
     my $name = shift;
@@ -439,7 +544,7 @@ sub _writeSymbolicLink {
 
 sub isSymbolicLink {
     my $self = shift;
-    if ( $self->{'externalFileAttributes'} == 2717843456 ) {
+    if ( $self->{'externalFileAttributes'} == 0xA1FF0000 ) {
         $self->{'isSymbolicLink'} = 1;
     } else {
         return 0;
@@ -489,16 +594,22 @@ sub _deflater {
 # Return the total size of my local header
 sub _localHeaderSize {
     my $self = shift;
-    return SIGNATURE_LENGTH + LOCAL_FILE_HEADER_LENGTH +
-      length( $self->fileName() ) + length( $self->localExtraField() );
+    {
+        use bytes;
+        return SIGNATURE_LENGTH + LOCAL_FILE_HEADER_LENGTH +
+          length( $self->fileName() ) + length( $self->localExtraField() );
+    }
 }
 
 # Return the total size of my CD header
 sub _centralDirectoryHeaderSize {
     my $self = shift;
-    return SIGNATURE_LENGTH + CENTRAL_DIRECTORY_FILE_HEADER_LENGTH +
-      length( $self->fileName() ) + length( $self->cdExtraField() ) +
-      length( $self->fileComment() );
+    {
+        use bytes;
+        return SIGNATURE_LENGTH + CENTRAL_DIRECTORY_FILE_HEADER_LENGTH +
+          length( $self->fileName() ) + length( $self->cdExtraField() ) +
+          length( $self->fileComment() );
+    }
 }
 
 # DOS date/time format
@@ -569,18 +680,22 @@ sub _writeLocalFileHeader {
     $self->_print($fh, $signatureData)
       or return _ioError("writing local header signature");
 
-    my $header = pack(
-        LOCAL_FILE_HEADER_FORMAT,
-        $self->versionNeededToExtract(),
-        $self->bitFlag(),
-        $self->desiredCompressionMethod(),
-        $self->lastModFileDateTime(),
-        $self->crc32(),
-        $self->compressedSize(),    # may need to be re-written later
-        $self->uncompressedSize(),
-        length( $self->fileName() ),
-        length( $self->localExtraField() )
-    );
+    my $header;
+    {
+        use bytes;
+        $header = pack(
+            LOCAL_FILE_HEADER_FORMAT,
+            $self->versionNeededToExtract(),
+            $self->bitFlag(),
+            $self->desiredCompressionMethod(),
+            $self->lastModFileDateTime(),
+            $self->crc32(),
+            $self->compressedSize(),    # may need to be re-written later
+            $self->uncompressedSize(),
+            length( $self->fileName() ),
+            length( $self->localExtraField() )
+        );
+    }
 
     $self->_print($fh, $header) or return _ioError("writing local header");
 
@@ -606,9 +721,13 @@ sub _writeCentralDirectoryFileHeader {
     $self->_print($fh, $sigData)
       or return _ioError("writing central directory header signature");
 
-    my $fileNameLength    = length( $self->fileName() );
-    my $extraFieldLength  = length( $self->cdExtraField() );
-    my $fileCommentLength = length( $self->fileComment() );
+    my ( $fileNameLength, $extraFieldLength, $fileCommentLength );
+    {
+        use bytes;
+        $fileNameLength    = length( $self->fileName() );
+        $extraFieldLength  = length( $self->cdExtraField() );
+        $fileCommentLength = length( $self->fileComment() );
+    }
 
     my $header = pack(
         CENTRAL_DIRECTORY_FILE_HEADER_FORMAT,
@@ -681,18 +800,22 @@ sub _refreshLocalFileHeader {
         IO::Seekable::SEEK_SET )
       or return _ioError("seeking to rewrite local header");
 
-    my $header = pack(
-        LOCAL_FILE_HEADER_FORMAT,
-        $self->versionNeededToExtract(),
-        $self->bitFlag(),
-        $self->desiredCompressionMethod(),
-        $self->lastModFileDateTime(),
-        $self->crc32(),
-        $self->_writeOffset(),    # compressed size
-        $self->uncompressedSize(),
-        length( $self->fileName() ),
-        length( $self->localExtraField() )
-    );
+    my $header;
+    {
+        use bytes;
+        $header = pack(
+            LOCAL_FILE_HEADER_FORMAT,
+            $self->versionNeededToExtract(),
+            $self->bitFlag(),
+            $self->desiredCompressionMethod(),
+            $self->lastModFileDateTime(),
+            $self->crc32(),
+            $self->_writeOffset(),    # compressed size
+            $self->uncompressedSize(),
+            length( $self->fileName() ),
+            length( $self->localExtraField() )
+        );
+    }
 
     $self->_print($fh, $header)
       or return _ioError("re-writing local header");
@@ -703,7 +826,8 @@ sub _refreshLocalFileHeader {
 }
 
 sub readChunk {
-    my ( $self, $chunkSize ) = @_;
+    my $self = shift;
+    my $chunkSize = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{chunkSize} : $_[0];
 
     if ( $self->readIsDone() ) {
         $self->endRead();
@@ -760,11 +884,11 @@ sub _copyChunk {
 # ( $outputRef, $status) = $self->_deflateChunk( \$buffer );
 sub _deflateChunk {
     my ( $self, $buffer ) = @_;
-    my ( $out,  $status ) = $self->_deflater()->deflate($buffer);
+    my ( $status ) = $self->_deflater()->deflate( $buffer, my $out );
 
     if ( $self->_readDataRemaining() == 0 ) {
         my $extraOutput;
-        ( $extraOutput, $status ) = $self->_deflater()->flush();
+        ( $status ) = $self->_deflater()->flush($extraOutput);
         $out .= $extraOutput;
         $self->endRead();
         return ( \$out, AZ_STREAM_END );
@@ -783,7 +907,7 @@ sub _deflateChunk {
 # ( $outputRef, $status) = $self->_inflateChunk( \$buffer );
 sub _inflateChunk {
     my ( $self, $buffer ) = @_;
-    my ( $out,  $status ) = $self->_inflater()->inflate($buffer);
+    my ( $status ) = $self->_inflater()->inflate( $buffer, my $out );
     my $retval;
     $self->endRead() unless $status == Z_OK;
     if ( $status == Z_OK || $status == Z_STREAM_END ) {
@@ -816,7 +940,7 @@ sub rewindData {
     if (    $self->compressionMethod() == COMPRESSION_STORED
         and $self->desiredCompressionMethod() == COMPRESSION_DEFLATED )
     {
-        ( $self->{'deflater'}, $status ) = Compress::Zlib::deflateInit(
+        ( $self->{'deflater'}, $status ) = Compress::Raw::Zlib::Deflate->new(
             '-Level'      => $self->desiredCompressionLevel(),
             '-WindowBits' => -MAX_WBITS(),                     # necessary magic
             '-Bufsize'    => $Archive::Zip::ChunkSize,
@@ -829,7 +953,7 @@ sub rewindData {
     elsif ( $self->compressionMethod() == COMPRESSION_DEFLATED
         and $self->desiredCompressionMethod() == COMPRESSION_STORED )
     {
-        ( $self->{'inflater'}, $status ) = Compress::Zlib::inflateInit(
+        ( $self->{'inflater'}, $status ) = Compress::Raw::Zlib::Inflate->new(
             '-WindowBits' => -MAX_WBITS(),               # necessary magic
             '-Bufsize'    => $Archive::Zip::ChunkSize,
             @_
@@ -916,7 +1040,7 @@ sub contents {
 sub extractToFileHandle {
     my $self = shift;
     return _error("encryption unsupported") if $self->isEncrypted();
-    my $fh = shift;
+    my $fh = ( ref( $_[0] ) eq 'HASH' ) ? shift->{fileHandle} : shift;
     _binmode($fh);
     my $oldCompression = $self->desiredCompressionMethod(COMPRESSION_STORED);
     my $status         = $self->rewindData(@_);
